namespace UnityEngine.UI.Windows.Components {

    public static class HTMLUtils {

        public static bool Has(HtmlAgilityPack.HtmlNode node, string attr) {
            var attrValue = node.Attributes[attr]?.Value;
            if (attrValue != null) {
                return true;
            }
            return false;
        }
        
        public static bool GetColor(HtmlAgilityPack.HtmlNode node, string attr, out Color value) {
            value = default;
            var attrValue = node.Attributes[attr]?.Value;
            if (attrValue != null) {
                value = GetColor(node, attr);
                return true;
            }
            return false;
        }

        public static Color GetColor(HtmlAgilityPack.HtmlNode node, string attr, Color defaultValue = default) {
            var attrValue = node.Attributes[attr]?.Value;
            if (attrValue != null) {
                if (ColorUtility.TryParseHtmlString(attrValue, out var color) == true) {
                    return color;
                } else {
                    {
                        var match = System.Text.RegularExpressions.Regex.Match(attrValue, @"rgba\((.*?),(.*?),(.*?),(.*?)\)",
                                                                               System.Text.RegularExpressions.RegexOptions.IgnoreCase);
                        if (match.Groups.Count > 1) {
                            byte.TryParse(match.Groups[1].Value, out var r);
                            byte.TryParse(match.Groups[2].Value, out var g);
                            byte.TryParse(match.Groups[3].Value, out var b);
                            float.TryParse(match.Groups[4].Value, out var a);
                            return new Color32(r, g, b, (byte)Mathf.Clamp(a * 255f, 0f, 255f));
                        }
                    }
                    {
                        var match = System.Text.RegularExpressions.Regex.Match(attrValue, @"rgb\((.*?),(.*?),(.*?)\)", System.Text.RegularExpressions.RegexOptions.IgnoreCase);
                        if (match.Groups.Count > 1) {
                            byte.TryParse(match.Groups[1].Value, out var r);
                            byte.TryParse(match.Groups[2].Value, out var g);
                            byte.TryParse(match.Groups[3].Value, out var b);
                            return new Color32(r, g, b, 255);
                        }
                    }
                    Debug.LogWarning($"Parse color failed: {attrValue}");
                }
            }
            return defaultValue;
        }
        
        public static bool GetEnum<T>(HtmlAgilityPack.HtmlNode node, string attr, out T value) where T : struct {
            value = default;
            var attrValue = node.Attributes[attr]?.Value;
            if (attrValue != null) {
                value = GetEnum<T>(node, attr);
                return true;
            }
            return false;
        }

        public static T GetEnum<T>(HtmlAgilityPack.HtmlNode node, string attr, T defaultValue = default) where T : struct {
            var attrValue = node.Attributes[attr]?.Value;
            if (System.Enum.TryParse<T>(attrValue, out var value) == true) {
                return value;
            }

            return defaultValue;
        }

        public static bool GetBool(HtmlAgilityPack.HtmlNode node, string attr, out bool value) {
            value = default;
            var attrValue = node.Attributes[attr]?.Value;
            if (attrValue != null) {
                value = GetBool(node, attr);
                return true;
            }
            return false;
        }

        public static bool GetBool(HtmlAgilityPack.HtmlNode node, string attr, bool defaultValue = default) {
            var attrValue = node.Attributes[attr]?.Value;
            if (bool.TryParse(attrValue, out var value) == true) {
                return value;
            }

            return defaultValue;
        }

        public static bool GetInt(HtmlAgilityPack.HtmlNode node, string attr, out int value) {
            value = default;
            var attrValue = node.Attributes[attr]?.Value;
            if (attrValue != null) {
                value = GetInt(node, attr);
                return true;
            }
            return false;
        }

        public static int GetInt(HtmlAgilityPack.HtmlNode node, string attr, int defaultValue = default) {
            var attrValue = node.Attributes[attr]?.Value;
            if (int.TryParse(attrValue, out var value) == true) {
                return value;
            }

            return defaultValue;
        }

        public static bool GetFloat(HtmlAgilityPack.HtmlNode node, string attr, out float value) {
            value = default;
            var attrValue = node.Attributes[attr]?.Value;
            if (attrValue != null) {
                value = GetFloat(node, attr);
                return true;
            }
            return false;
        }

        public static float GetFloat(HtmlAgilityPack.HtmlNode node, string attr, float defaultValue = default) {
            var attrValue = node.Attributes[attr]?.Value;
            if (float.TryParse(attrValue, out var value) == true) {
                return value;
            }

            return defaultValue;
        }

        public static float[] GetFloatArray(HtmlAgilityPack.HtmlNode node, string attr, float[] defaultValue = default) {
            var attrValue = node.Attributes[attr]?.Value;
            if (attrValue != null) {
                var values = attrValue.Split(',');
                var arr = new float[values.Length];
                for (int i = 0; i < arr.Length; ++i) {
                    if (float.TryParse(values[i], out var value) == true) {
                        arr[i] = value;
                    }
                }

                return arr;
            }

            return defaultValue;
        }

        public static bool GetString(HtmlAgilityPack.HtmlNode node, string attr, out string value) {
            value = default;
            var attrValue = node.Attributes[attr]?.Value;
            if (attrValue != null) {
                value = GetString(node, attr);
                return true;
            }
            return false;
        }

        public static string GetString(HtmlAgilityPack.HtmlNode node, string attr, string defaultValue = default) {
            var attrValue = node.Attributes[attr]?.Value;
            if (attrValue != null) {
                return attrValue;
            }

            return defaultValue;
        }

    }

}